@@ -2,10 +2,11 @@
 #include "../Common/Common.h"
 
 #include "AlphabetManager.h"
-#include "DasherModel.h"
+//#include "DasherModel.h"
 #include "DasherNode.h"
 #include "Event.h"
 #include "EventHandler.h"
+#include "NodeCreationManager.h"
 
 #include <vector>
 #include <iostream>
@@ -22,9 +23,10 @@ static char THIS_FILE[] = __FILE__;
 #endif
 #endif
 
-CAlphabetManager::CAlphabetManager( CDasherModel *pModel, CLanguageModel *pLanguageModel, bool bGameMode, const std::string &strGameModeText ) 
-  : CNodeManager(0), m_pLanguageModel(pLanguageModel), m_pModel(pModel) {
+CAlphabetManager::CAlphabetManager( CNodeCreationManager *pNCManager, CLanguageModel *pLanguageModel, CLanguageModel::Context iLearnContext, bool bGameMode, const std::string &strGameModeText ) 
+  : CNodeManager(0), m_pLanguageModel(pLanguageModel), m_pNCManager(pNCManager) {
 
+  m_iLearnContext = iLearnContext;
   m_bGameMode = bGameMode;
   m_strGameString = strGameModeText;
 }
@@ -46,15 +48,15 @@ CDasherNode *CAlphabetManager::GetRoot(CDasherNode *pParent, int iLower, int iUp
   if(iSymbol == 0)
     iColour = 7;
   else
-    iColour = m_pModel->GetColour(iSymbol);
+    iColour = m_pNCManager->GetColour(iSymbol);
 
   pNewNode = new CDasherNode(pParent, iSymbol, 0, Opts::Nodes1, iLower, iUpper, m_pLanguageModel, iColour);
   
-  pNewNode->SetContext(m_pLanguageModel->CreateEmptyContext()); // FIXME - handle context properly
+  //  pNewNode->SetContext(m_pLanguageModel->CreateEmptyContext()); // FIXME - handle context properly
   pNewNode->m_pNodeManager = this;
   pNewNode->m_bShove = true;
-  pNewNode->m_pBaseGroup = m_pModel->GetAlphabet().m_pBaseGroup;
-  pNewNode->m_strDisplayText = m_pModel->GetAlphabet().GetDisplayText(iSymbol);
+  pNewNode->m_pBaseGroup = m_pNCManager->GetAlphabet()->m_pBaseGroup;
+  pNewNode->m_strDisplayText = m_pNCManager->GetAlphabet()->GetDisplayText(iSymbol);
   pNewNode->Seen(true);
 
   SAlphabetData *pNodeUserData = new SAlphabetData;
@@ -62,6 +64,14 @@ CDasherNode *CAlphabetManager::GetRoot(CDasherNode *pParent, int iLower, int iUp
 
   pNodeUserData->iPhase = 0;
   pNodeUserData->iSymbol = iSymbol;
+  pNodeUserData->pLanguageModel = m_pLanguageModel;
+
+  // Hopefully this will obsolete any need to handle contexts outside
+  // of the alphabet manager - check this and remove resulting
+  // redundant code
+  CLanguageModel::Context iContext;
+  iContext = m_pLanguageModel->CreateEmptyContext();
+  pNodeUserData->iContext = iContext;
 
   if(m_bGameMode) {
     pNodeUserData->iGameOffset = -1;
@@ -79,6 +89,7 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
   int iOldPhase(static_cast<SAlphabetData *>(pNode->m_pUserData)->iPhase);
   int iNewPhase((iOldPhase + 1) % 2);
 
+  SAlphabetData *pParentUserData(static_cast<SAlphabetData *>(pNode->m_pUserData));
 
   // Actually create the children here
   
@@ -86,7 +97,8 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
   std::vector < symbol > newchars; // place to put this list of characters
   std::vector < unsigned int >cum; // for the probability list
   
-  m_pModel->GetProbs(pNode->Context(), newchars, cum, m_pModel->GetLongParameter(LP_NORMALIZATION));
+  // TODO: Need to fix up relation to language model here (use one from node, not global).
+  m_pNCManager->GetProbs(pParentUserData->iContext, newchars, cum, m_pNCManager->GetLongParameter(LP_NORMALIZATION));
   int iChildCount = newchars.size();
   
   // work out cumulative probs in place
@@ -100,30 +112,26 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
   for(int j = 0; j < iChildCount; j++) {
     CDasherNode *pNewNode;
     
-    if(newchars[j] == m_pModel->GetControlSymbol())
-      pNewNode = m_pModel->GetRoot(1, pNode, iLbnd, cum[j], NULL);
-    else if(newchars[j] == m_pModel->GetStartConversionSymbol()) {
-#ifdef JAPANESE
-      pNewNode = m_pModel->GetRoot(2, pNode, iLbnd, cum[j], NULL);
-#else
-      pNewNode = m_pModel->GetRoot(0, pNode, iLbnd, cum[j], NULL);
-      pNewNode->Seen(false);
-#endif
+    if(newchars[j] == m_pNCManager->GetControlSymbol())
+      pNewNode = m_pNCManager->GetRoot(1, pNode, iLbnd, cum[j], NULL);
+    else if(newchars[j] == m_pNCManager->GetStartConversionSymbol()) {
+      // TODO: Need to consider the case where there is no compile-time support for this
+      pNewNode = m_pNCManager->GetRoot(2, pNode, iLbnd, cum[j], NULL);
     }
     else if( newchars[j] == iExistingSymbol) {
       pNewNode = pExistingChild;
       pNewNode->SetRange(iLbnd, cum[j]);
     }
     else {
-      int iColour(m_pModel->GetColour(newchars[j]));
+      int iColour(m_pNCManager->GetColour(newchars[j]));
 
       // This is provided for backwards compatibility. 
       // Colours should always be provided by the alphabet file
       if(iColour == -1) {
-	if(newchars[j] == m_pModel->GetSpaceSymbol()) {
+	if(newchars[j] == m_pNCManager->GetSpaceSymbol()) {
 	  iColour = 9;
 	}
-	else if(newchars[j] == m_pModel->GetControlSymbol()) {
+	else if(newchars[j] == m_pNCManager->GetControlSymbol()) {
 	  iColour = 8;
 	}
 	else {
@@ -135,10 +143,10 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
       if(iNewPhase == 1 && iColour < 130) // We don't loop on high
 	iColour += 130;
       
-      pNewNode = new CDasherNode(pNode, newchars[j], 0, Nodes1, iLbnd, cum[j], m_pLanguageModel, iColour);
+      pNewNode = new CDasherNode(pNode, newchars[j], 0, Nodes1, iLbnd, cum[j], pParentUserData->pLanguageModel, iColour);
       pNewNode->m_pNodeManager = this;
       pNewNode->m_bShove = true;
-      pNewNode->m_pBaseGroup = m_pModel->GetAlphabet().m_pBaseGroup;
+      pNewNode->m_pBaseGroup = m_pNCManager->GetAlphabet()->m_pBaseGroup;
       
       SAlphabetData *pNodeUserData = new SAlphabetData;
       pNewNode->m_pUserData = pNodeUserData;
@@ -149,18 +157,26 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
       if(m_bGameMode) {
 	int iCurrentGameOffset(static_cast<SAlphabetData *>(pNode->m_pUserData)->iGameOffset);
 	
-	if((iCurrentGameOffset != -2) && ((iCurrentGameOffset + 1) < m_strGameString.size()) 
-	   && ((m_pModel->GetAlphabet().GetText(newchars[j]))[0] == m_strGameString[iCurrentGameOffset + 1])) {
+	if((iCurrentGameOffset != -2) && ((iCurrentGameOffset + 1) < static_cast<int>(m_strGameString.size())) 
+	   && ((m_pNCManager->GetAlphabet()->GetText(newchars[j]))[0] == m_strGameString[iCurrentGameOffset + 1])) {
 	  static_cast<SAlphabetData *>(pNewNode->m_pUserData)->iGameOffset = iCurrentGameOffset + 1;
 	  pNewNode->SetGame(true);
 	}
 	else
 	  static_cast<SAlphabetData *>(pNewNode->m_pUserData)->iGameOffset = -2;
       }
+
+      CLanguageModel::Context iContext;
+      iContext = m_pLanguageModel->CloneContext(pParentUserData->iContext);
+      m_pLanguageModel->EnterSymbol(iContext, newchars[j]); // TODO: Don't use symbols?
+      
+      pNodeUserData->pLanguageModel = pParentUserData->pLanguageModel;
+      pNodeUserData->iContext = iContext;
+      
+      //      pNewNode->SetContext(iContext); // FIXME - handle context properly
     }
     
-    pNewNode->m_strDisplayText = m_pModel->GetAlphabet().GetDisplayText(newchars[j]);
-    
+    pNewNode->m_strDisplayText = m_pNCManager->GetAlphabet()->GetDisplayText(newchars[j]);
     
     pNode->Children()[j] = pNewNode;
     iLbnd = cum[j];
@@ -170,16 +186,19 @@ void CAlphabetManager::PopulateChildrenWithSymbol( CDasherNode *pNode, int iExis
 }
 
 void CAlphabetManager::ClearNode( CDasherNode *pNode ) {
-  delete static_cast<SAlphabetData *>(pNode->m_pUserData);
+  SAlphabetData *pUserData(static_cast<SAlphabetData *>(pNode->m_pUserData));
+  
+  pUserData->pLanguageModel->ReleaseContext(pUserData->iContext);
+  delete pUserData;
 }
 
 void CAlphabetManager::Output( CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int iNormalization) {
-  m_pModel->m_bContextSensitive = true;
+  //  m_pNCManager->m_bContextSensitive = true;
   symbol t = static_cast<SAlphabetData *>(pNode->m_pUserData)->iSymbol;
 
   if(t) { // Ignore symbol 0 (root node)
-    Dasher::CEditEvent oEvent(1, m_pModel->GetAlphabet().GetText(t));
-    m_pModel->InsertEvent(&oEvent);
+    Dasher::CEditEvent oEvent(1, m_pNCManager->GetAlphabet()->GetText(t));
+    m_pNCManager->InsertEvent(&oEvent);
 
     // Track this symbol and its probability for logging purposes
     if (pAdded != NULL) {
@@ -195,22 +214,25 @@ void CAlphabetManager::Output( CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* p
 void CAlphabetManager::Undo( CDasherNode *pNode ) {
   symbol t = static_cast<SAlphabetData *>(pNode->m_pUserData)->iSymbol;
   if(t) { // Ignore symbol 0 (root node)
-    Dasher::CEditEvent oEvent(2, m_pModel->GetAlphabet().GetText(t));
-    m_pModel->InsertEvent(&oEvent);
+    Dasher::CEditEvent oEvent(2, m_pNCManager->GetAlphabet()->GetText(t));
+    m_pNCManager->InsertEvent(&oEvent);
   }
 }
 
 CDasherNode *CAlphabetManager::RebuildParent(CDasherNode *pNode, int iGeneration) {
 
+  // TODO: Reimplement -----
+
   // Ask for context information
-  m_pModel->m_strContextBuffer = "";
-  Dasher::CEditContextEvent oEvent(10);
-  m_pModel->InsertEvent(&oEvent);
+  //  m_pNCManager->m_strContextBuffer = "";
+  //  Dasher::CEditContextEvent oEvent(10);
+  //  m_pNCManager->InsertEvent(&oEvent);
 
-  std::string strContext(m_pModel->m_strContextBuffer);
+  //  std::string strContext(m_pNCManager->m_strContextBuffer);
   std::vector<symbol> vSymbols;
-  m_pLanguageModel->SymbolAlphabet().GetAlphabetPointer()->GetSymbols(&vSymbols, &strContext, false);
+  //  m_pLanguageModel->SymbolAlphabet().GetAlphabetPointer()->GetSymbols(&vSymbols, &strContext, false);
 
+  // -----
 
   CDasherNode *pNewNode;
 
@@ -219,7 +241,9 @@ CDasherNode *CAlphabetManager::RebuildParent(CDasherNode *pNode, int iGeneration
 
   symbol iNewSymbol;
 
-  if(vSymbols.size() <= iGeneration + 1) {
+  CLanguageModel::Context oContext(m_pLanguageModel->CreateEmptyContext());
+
+  if(static_cast<int>(vSymbols.size()) <= iGeneration + 1) {
     // Create a root node if there isn't any more context
     iNewPhase = 0;
     iNewSymbol = 0;
@@ -227,40 +251,37 @@ CDasherNode *CAlphabetManager::RebuildParent(CDasherNode *pNode, int iGeneration
     pNewNode = new CDasherNode(0, 0, 0,  Opts::Nodes1, 0, 0, m_pLanguageModel, 7);
     
     // TODO: Horrible hard coded default context
-    CLanguageModel::Context oContext(m_pLanguageModel->CreateEmptyContext());
-    m_pModel->EnterText(oContext, ". ");
-    pNewNode->SetContext(oContext);
+    m_pNCManager->EnterText(oContext, ". ");
   }
   else {
     
     iNewPhase = ((iOldPhase + 2 - 1) % 2);
     iNewSymbol = vSymbols[vSymbols.size() - iGeneration - 2];
 
-    int iColour(m_pModel->GetColour(vSymbols[vSymbols.size() - iGeneration - 2]));
+    int iColour(m_pNCManager->GetColour(vSymbols[vSymbols.size() - iGeneration - 2]));
 
     // Loop colours if necessary for the colour scheme
     if(iNewPhase == 1)
       iColour += 130;
             
     pNewNode = new CDasherNode(0, iNewSymbol, 0, Nodes1, 0, 0, m_pLanguageModel, iColour);
-    CLanguageModel::Context oContext(m_pLanguageModel->CreateEmptyContext());
-    
-    for(int i(0); i < vSymbols.size() - iGeneration -1; ++i)
+
+    for(unsigned int i(0); i < vSymbols.size() - iGeneration -1; ++i)
       m_pLanguageModel->EnterSymbol(oContext, vSymbols[i]);
-    
-    pNewNode->SetContext(oContext);
   }
   
   pNewNode->m_pNodeManager = this;
   pNewNode->m_bShove = true;
   pNewNode->Seen(true);
-  pNewNode->m_pBaseGroup = m_pModel->GetAlphabet().m_pBaseGroup;
+  pNewNode->m_pBaseGroup = m_pNCManager->GetAlphabet()->m_pBaseGroup;
 
   SAlphabetData *pNodeUserData = new SAlphabetData;
   pNewNode->m_pUserData = pNodeUserData;
 
   pNodeUserData->iPhase = iNewPhase;
   pNodeUserData->iSymbol = iNewSymbol;
+  pNodeUserData->pLanguageModel = m_pLanguageModel;
+  pNodeUserData->iContext = oContext;
 
   PopulateChildrenWithSymbol(pNewNode, static_cast<SAlphabetData *>(pNode->m_pUserData)->iSymbol, pNode);
 
@@ -268,3 +289,13 @@ CDasherNode *CAlphabetManager::RebuildParent(CDasherNode *pNode, int iGeneration
   
   return pNewNode;
 }
+
+void CAlphabetManager::SetFlag(CDasherNode *pNode, int iFlag, bool bValue) {
+  switch(iFlag) {
+  case NF_COMMITTED:
+    if(bValue)
+      // TODO: Reimplement (need a learning context, check whether symbol actually corresponds to character)
+      static_cast<SAlphabetData *>(pNode->m_pUserData)->pLanguageModel->LearnSymbol(m_iLearnContext, static_cast<SAlphabetData *>(pNode->m_pUserData)->iSymbol);
+    break;
+  }
+}
