@@ -52,9 +52,16 @@ void CDasherView::ChangeScreen(CDasherScreen *NewScreen) {
 
 /////////////////////////////////////////////////////////////////////////////
 
-bool CDasherView::Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, bool bRedrawDisplay) {
+bool CDasherView::Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, bool bRedrawDisplay, bool bGameMode) {
+
+  m_iRenderCount = 0;
+
+  myint iGamePointer;
+  RenderNodes(pRoot, iRootMin, iRootMax, vNodeList, vDeleteList, &iGamePointer);
+
+  if(bGameMode)
+    DrawGameModePointer(iGamePointer);
 
-  RenderNodes(pRoot, iRootMin, iRootMax, vNodeList, vDeleteList);
   return true;
 }
 
@@ -66,7 +73,7 @@ int CDasherView::GetCoordinateCount() {
     return 0;
 }
 
-int CDasherView::GetCoordinates(int iN, myint * pCoordinates) {
+int CDasherView::GetInputCoordinates(int iN, myint * pCoordinates) {
   if(m_pInput)
     return m_pInput->GetCoordinates(iN, pCoordinates);
   else
