@@ -7,8 +7,6 @@
 
 using namespace Dasher;
 
-extern "C" gint canvas_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data);
-
 CCanvas::CCanvas(GtkWidget *pCanvas, CPangoCache *pPangoCache)
   : CDasherScreen(pCanvas->allocation.width, pCanvas->allocation.height) {
 
@@ -27,6 +25,13 @@ CCanvas::CCanvas(GtkWidget *pCanvas, CPangoCache *pPangoCache)
   // Construct the buffer pixmaps
   // FIXME - only allocate without cairo
 
+#if WITH_CAIRO
+
+  m_pDisplaySurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
+  m_pDecorationSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
+  m_pOnscreenSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
+
+#else
 
   m_pDummyBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
 
@@ -38,29 +43,29 @@ CCanvas::CCanvas(GtkWidget *pCanvas, CPangoCache *pPangoCache)
 
   m_pOffscreenBuffer = m_pDisplayBuffer;
 
+#endif
+
 
 #if WITH_CAIRO
   // The lines between origin and pointer is draw here
-  decoration_cr = gdk_cairo_create(m_pDecorationBuffer);
+  decoration_cr = cairo_create(m_pDecorationSurface);
   cr = decoration_cr;
   cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
   cairo_set_line_width(cr, 1.0);
 
   // Base stuff are drawn here
-  display_cr = gdk_cairo_create(m_pDisplayBuffer);
+  display_cr = cairo_create(m_pDisplaySurface);
   cr = display_cr;
   cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
   cairo_set_line_width(cr, 1.0);
+
+  onscreen_cr = cairo_create(m_pOnscreenSurface);
+
+  widget_cr = gdk_cairo_create(m_pCanvas->window) ;
+
 #endif
 
   m_pPangoInk = new PangoRectangle;
-
-  lSignalHandler = g_signal_connect(m_pCanvas, "expose_event", G_CALLBACK(canvas_expose_event), this);
-
-/*  gtk_widget_add_events(m_pCanvas, GDK_EXPOSURE_MASK);
-  gtk_widget_add_events(m_pCanvas, GDK_BUTTON_PRESS_MASK);
-  gtk_widget_add_events(m_pCanvas, GDK_BUTTON_RELEASE_MASK);
-*/
   gtk_widget_add_events(m_pCanvas, GDK_ALL_EVENTS_MASK);
 }
 
@@ -71,14 +76,12 @@ CCanvas::~CCanvas() {
   cr = NULL;
   cairo_destroy(display_cr);
   cairo_destroy(decoration_cr);
-#endif
-
+#else
   g_object_unref(m_pDummyBuffer);
   g_object_unref(m_pDisplayBuffer);
   g_object_unref(m_pDecorationBuffer);
   g_object_unref(m_pOnscreenBuffer);
-
-  g_signal_handler_disconnect(m_pCanvas, lSignalHandler);
+#endif
 
   delete m_pPangoInk;
 }
@@ -108,10 +111,8 @@ void CCanvas::Blank() {
 }
 
 void CCanvas::Display() {
-
   // FIXME - Some of this stuff is probably not needed
-
-  GdkRectangle update_rect;
+  //  GdkRectangle update_rect;
 
   GdkGC *graphics_context;
 #if WITH_CAIRO
@@ -131,26 +132,53 @@ void CCanvas::Display() {
 
   // Copy the offscreen buffer into the onscreen buffer
 
-  gdk_draw_drawable(m_pOnscreenBuffer, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pOffscreenBuffer, 0, 0, 0, 0, m_iWidth, m_iHeight);
+  // TODO: Reimplement (kind of important!)
+
+  //  gdk_draw_drawable(m_pOnscreenBuffer, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pOffscreenBuffer, 0, 0, 0, 0, m_iWidth, m_iHeight);
   
+  //  BEGIN_DRAWING;
+
+//   cairo_set_source_surface(onscreen_cr, m_pDecorationSurface, 0, 0);
+//   cairo_rectangle(onscreen_cr, 0, 0, m_iWidth, m_iHeight);
+//   cairo_fill(onscreen_cr);
+
+
+
+  //  END_DRAWING;
+
   // Blank the offscreen buffer (?)
 
-  gdk_draw_rectangle(m_pOffscreenBuffer, graphics_context, TRUE, 0, 0, m_iWidth, m_iHeight);
+  //  gdk_draw_rectangle(m_pOffscreenBuffer, graphics_context, TRUE, 0, 0, m_iWidth, m_iHeight);
 
   // Invalidate the full canvas to force it to be redrawn on-screen
 
-  update_rect.x = 0;
-  update_rect.y = 0;
-  update_rect.width = m_iWidth;
-  update_rect.height = m_iHeight;
+ //  update_rect.x = 0;
+//   update_rect.y = 0;
+//   update_rect.width = m_iWidth;
+//   update_rect.height = m_iHeight;
 
-  gdk_window_invalidate_rect(m_pCanvas->window, &update_rect, FALSE);
+//   gdk_window_invalidate_rect(m_pCanvas->window, &update_rect, FALSE);
+
+  //  BEGIN_DRAWING;
+
+  //  GdkRectangle sRect = {0, 0, m_iWidth, m_iHeight};
+  //  gdk_window_begin_paint_rect(m_pCanvas->window, &sRect);
+
+#if WITH_CAIRO  
+  cairo_set_source_surface(widget_cr, m_pDecorationSurface, 0, 0);
+  cairo_rectangle(widget_cr, 0, 0, m_iWidth, m_iHeight);
+  cairo_fill(widget_cr);
+#else
+  gdk_draw_drawable(m_pCanvas->window, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pDecorationBuffer, 0, 0, 0, 0, m_iWidth, m_iHeight);
+#endif
+
+  //   gdk_window_end_paint(m_pCanvas->window);
 
   // Restore original graphics context (?)
 
   END_DRAWING;
 
-  gtk_main_iteration_do(0);
+  // gtk_main_iteration_do(0);
 }
 
 void CCanvas::DrawRectangle(int x1, int y1, int x2, int y2, int Color, int iOutlineColour, Opts::ColorSchemes ColorScheme, bool bDrawOutline, bool bFill, int iThickness) {
@@ -218,9 +246,12 @@ void CCanvas::DrawRectangle(int x1, int y1, int x2, int y2, int Color, int iOutl
   END_DRAWING;
 }
 
-void CCanvas::DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour, bool bFill) {
+void CCanvas::DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour, int iFillColour, int iThickness, bool bFill) {
 #if WITH_CAIRO
 #else
+  if(iThickness == 1) // This is to make it work propely on Windows
+    iThickness = 0; 
+
   GdkGC *graphics_context;
   GdkColormap *colormap;
 
@@ -231,7 +262,7 @@ void CCanvas::DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour
   BEGIN_DRAWING;
 
   if(bFill) {
-    SET_COLOR(iColour);
+    SET_COLOR(iFillColour);
 #if WITH_CAIRO
     cairo_arc(cr, iCX, iCY, iR, 0, 2*M_PI);
     cairo_fill(cr);
@@ -240,11 +271,13 @@ void CCanvas::DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour
 #endif
   }
 
-  SET_COLOR(2);
+  SET_COLOR(iColour);
 #if WITH_CAIRO
+  cairo_set_line_width(cr, iThickness);
   cairo_arc(cr, iCX, iCY, iR, 0, 2*M_PI);
   cairo_stroke(cr);
 #else
+  gdk_gc_set_line_attributes(graphics_context, iThickness, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND );
   gdk_draw_arc(m_pOffscreenBuffer, graphics_context, false, iCX - iR, iCY - iR, 2*iR, 2*iR, 0, 23040);
 #endif
 
@@ -382,14 +415,24 @@ void CCanvas::SendMarker(int iMarker) {
 
   switch(iMarker) {
   case 0: // Switch to display buffer
-    m_pOffscreenBuffer = m_pDisplayBuffer;
 #if WITH_CAIRO
     cr = display_cr;
+#else
+    m_pOffscreenBuffer = m_pDisplayBuffer;
 #endif
     break;
   case 1: // Switch to decorations buffer
+
+#if WITH_CAIRO
+    BEGIN_DRAWING;
+    cairo_set_source_surface(decoration_cr, m_pDisplaySurface, 0, 0);
+    cairo_rectangle(decoration_cr, 0, 0, m_iWidth, m_iHeight);
+    cairo_fill(decoration_cr);
+    END_DRAWING;
+#else
     gdk_draw_drawable(m_pDecorationBuffer, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pDisplayBuffer, 0, 0, 0, 0, m_iWidth, m_iHeight);
     m_pOffscreenBuffer = m_pDecorationBuffer;
+#endif
 #if WITH_CAIRO
     cr = decoration_cr;
 #endif
@@ -397,13 +440,8 @@ void CCanvas::SendMarker(int iMarker) {
   }
 }
 
-bool CCanvas::ExposeEvent(GtkWidget *pWidget, GdkEventExpose *pEvent) {
-  gdk_draw_drawable(m_pCanvas->window, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pOnscreenBuffer, pEvent->area.x, pEvent->area.y, pEvent->area.x, pEvent->area.y, pEvent->area.width, pEvent->area.height);
-  return true;
-}
-
-void CCanvas::SetColourScheme(const Dasher::CCustomColours *Colours) {
-  int iNumColours(Colours->GetNumColours());
+void CCanvas::SetColourScheme(const CColourIO::ColourInfo *pColourScheme) {
+  int iNumColours(pColourScheme->Reds.size());
 
 #if WITH_CAIRO
   if (cairo_colours)
@@ -417,14 +455,14 @@ void CCanvas::SetColourScheme(const Dasher::CCustomColours *Colours) {
 
   for(int i = 0; i < iNumColours; i++) {
 #if WITH_CAIRO
-    cairo_colours[i].r = Colours->GetRed(i) / 255.0;
-    cairo_colours[i].g = Colours->GetGreen(i) / 255.0;
-    cairo_colours[i].b = Colours->GetBlue(i) / 255.0;
+    cairo_colours[i].r = pColourScheme->Reds[i] / 255.0;
+    cairo_colours[i].g = pColourScheme->Greens[i] / 255.0;
+    cairo_colours[i].b = pColourScheme->Blues[i] / 255.0;
 #else
     colours[i].pixel=0;
-    colours[i].red=Colours->GetRed(i)*257;
-    colours[i].green=Colours->GetGreen(i)*257;
-    colours[i].blue=Colours->GetBlue(i)*257;
+    colours[i].red=pColourScheme->Reds[i]*257;
+    colours[i].green=pColourScheme->Greens[i]*257;
+    colours[i].blue=pColourScheme->Blues[i]*257;
 #endif
   }
 }
@@ -449,8 +487,3 @@ bool CCanvas::GetCanvasSize(GdkRectangle *pRectangle)
 
   return true;
 }
-
-extern "C" gint canvas_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
-  return ((CCanvas*)data)->ExposeEvent(widget, event);
-}
-
